import abc
from typing import Type, List
from pydantic import BaseModel
from starlette.datastructures import UploadFile

from pymultirole_plugins.schema import FormDataModel, Document


class ConverterParameters(FormDataModel):
    pass


class ConverterBase(metaclass=abc.ABCMeta):
    """Base class for example plugin used in the tutorial.
    """

    def __init__(self):
        pass

    @abc.abstractmethod
    def convert(self, source: UploadFile, parameters: ConverterParameters) \
            -> List[Document]:
        """Parse the input source file and return a list of documents.

        :param source: An UploadFile containing the data.
        :param parameters: options of the converter.
        :returns: List of documents.
        """

    @classmethod
    def get_model(cls) -> Type[BaseModel]:
        return ConverterParameters



from typing import Type

import pytest
from pydantic import BaseModel, Field

from pymultirole_plugins.annotator import AnnotatorBase, AnnotatorParameters
from pymultirole_plugins.schema import Document, Category


def test_annotator():
    with pytest.raises(TypeError) as err:
        parser = AnnotatorBase()
        assert parser is None
    assert "Can't instantiate abstract class AnnotatorBase with abstract methods annotate" in str(err.value)


def test_default_options():
    options = AnnotatorParameters()
    assert options is not None


class DummyParameters(AnnotatorParameters):
    foo: str = Field("foo", description="Foo")
    bar: float = Field(0.123456789, description="Bar")


class Dummyannotator(AnnotatorBase):
    """Dummy annotator.
    """

    def annotate(self, document: Document, parameters: AnnotatorParameters) \
            -> Document:
        parameters: DummyParameters = parameters
        document.categories = [Category(labelName=parameters.foo, score=parameters.bar)]
        return document

    @classmethod
    def get_model(cls) -> Type[BaseModel]:
        return DummyParameters


def test_dummy():
    annotator = Dummyannotator()
    options = DummyParameters()
    doc: Document = annotator.annotate(Document(text="This is a test document", metadata=options.dict()), options)
    assert len(doc.categories) == 1
    assert doc.categories[0].labelName == options.foo
    assert doc.categories[0].score == options.bar



import json
from typing import Optional, List, Any, Dict

from pydantic import BaseModel, Field


class FormDataModel(BaseModel):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate_to_json

    @classmethod
    def validate_to_json(cls, value):
        if isinstance(value, str):
            return cls(**json.loads(value))
        return value


class Span(BaseModel):
    start: int = Field(..., description="Start index of the span in the text", example=5)
    end: int = Field(..., description="End index of the span in the text", example=15)


class Boundary(Span):
    name: str = Field(None, description="Name of the boundary", example="body")


class Term(BaseModel):
    identifier: str = Field(..., description="Unique identifier of the term", example="http://www.example.com/rocks")
    lexicon: str = Field(None, description="Lexicon of the term", example="MeSH")
    preferredForm: Optional[str] = Field(None, description="The preferred label of the term", example="rocks")

    # properties: Optional[Dict[str, Any]] = Field(None, description="Additional properties of the term",
    #                                              example={"altForms": ["basalt", "granite", "slate"],
    #                                                       "wikidataId": "Q8063"})
    def __eq__(self, other):
        return other and self.lexicon == other.lexicon and self.identifier == other.identifier

    def __hash__(self):
        return hash((self.identifier, self.lexicon))


class Annotation(Span):
    labelName: str = Field(None, description="Label name of the annotation", example="org")
    label: Optional[str] = Field(None, description="Label of the annotation", example="ORG")
    score: Optional[float] = Field(1.0, description="Confidence score of the annotation", example=0.87)
    properties: Optional[Dict[str, Any]] = Field(None, description="Properties of annotation")
    terms: Optional[List[Term]] = Field(None, description="Properties of annotation")


class Category(BaseModel):
    labelName: str = Field(None, description="Label name of the category", example="org")
    label: Optional[str] = Field(None, description="Label of the category", example="ORG")
    score: Optional[float] = Field(1.0, description="Confidence score of the category", example=0.87)
    properties: Optional[Dict[str, Any]] = Field(None, description="Properties of category")


class Document(BaseModel):
    text: str = Field(None, description="Plain text of the converted document")
    sourceText: Optional[str] = Field(None, description="Source text of the converted document")
    boundaries: Optional[Dict[str, List[Boundary]]] = Field(None, description="List of boundaries by type")
    metadata: Optional[Dict[str, Any]] = Field(None, description="Document metadata")
    properties: Optional[Dict[str, Any]] = Field(None, description="Document properties")
    sentences: Optional[List[Span]] = Field(None, description="Document sentences")
    annotations: Optional[List[Annotation]] = Field(None, description="Document annotations")
    categories: Optional[List[Category]] = Field(None, description="Document categories")



import abc
from typing import Type

from pydantic import BaseModel

from pymultirole_plugins.schema import Document


class AnnotatorParameters(BaseModel):
    pass


class AnnotatorBase(metaclass=abc.ABCMeta):
    """Base class for example plugin used in the tutorial.
    """

    def __init__(self):
        pass

    @abc.abstractmethod
    def annotate(self, document: Document, options: AnnotatorParameters) \
            -> Document:
        """Annotate the input document and return a modified document.

        :param document: An annotated document.
        :param options: options of the parser.
        :returns: Document.
        """

    @classmethod
    def get_model(cls) -> Type[BaseModel]:
        return AnnotatorParameters
